package com.bwie.auth.service.impl;

import ch.qos.logback.core.util.TimeUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.RandomUtils;
import com.bwie.auth.feign.AuthFieign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.UserReq;
import com.bwie.common.domain.response.JwtResp;
import com.bwie.common.exception.ProductException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.common.utils.TelSmsUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

/**
 * @Author：weiran
 * @Package：com.bwie.auth.service.impl
 * @Project：monthA8.12
 * @name：AuthServiceImpl
 * @Date：2024/8/12 9:24
 */
@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private AuthFieign authFieign;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;

    /**
     * 发送验证码
     */
    @Override
    public String sendcode(String userPhone) {
        //判断手机号是否为空
        if(StringUtils.isBlank(userPhone)){
            throw new ProductException("手机号不能为空");
        }
        //手机号是否存在
        Result<User> result = authFieign.findByuserPhone(userPhone);
        User login = result.getData();
        if (login==null){
            throw new ProductException("手机号不存在");
        }
        //创建验证码
        String code = RandomUtil.randomNumbers(4);
        //将验证码存入redis
        redisTemplate.opsForValue().set(userPhone, code,5, TimeUnit.MINUTES);
        //发送验证码
        HashMap<String, String> map = new HashMap<>();
        map.put(userPhone,code);
        TelSmsUtils.sendSms(userPhone, code, map);
        //返回code
        return code;
    }

    /**
     * 登录
     * @param userReq
     * @return
     */
    @Override
    public JwtResp login(UserReq userReq) {
        //判断手机号或验证码是否为空
        String userPhone = userReq.getUserPhone();
        String code = userReq.getCode();
        if (StringUtils.isAnyBlank(userPhone,code)){
            throw new ProductException("手机号或验证码不能为空");
        }
        //判断手机号是否存在  用户点击登录后，若系统中无此账号则需要先录入到系统用户表当中
        Result<User> result = authFieign.findByuserPhone(userPhone);
        User login = result.getData();
        if (login==null){
            String num = RandomUtil.randomNumbers(4);
            User user = new User();
            user.setUserName("游客"+num);
            user.setUserPhone(userPhone);
            authFieign.adduser(user);
            throw new ProductException("手机号不存在，已经以"+user.getUserName()+"的身份帮您录入,请重新登录");
        }
        //判断验证码是否过期
        Boolean aBoolean = redisTemplate.hasKey(userPhone);
        if (!aBoolean){
            throw new ProductException("验证码已过期");
        }
        //判断验证码是否正确
        String code1 = redisTemplate.opsForValue().get(userPhone);
        if (!code1.equals(code)){
            throw new ProductException("验证码错误");
        }
        //登录
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY, login.getUserId());
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+login.getUserId(), JSONObject.toJSONString(login),30, TimeUnit.MINUTES);
        JwtResp jwtResp = new JwtResp(token, "30MIN");
        return jwtResp;
    }

    /**
     * 身份验证
     * @return
     */
    @Override
    public User getInfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String code = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return JSONObject.parseObject(code, User.class);

    }
}
