package com.bwie.coupon.controller;

import com.bwie.common.domain.request.CouponBuyReq;
import com.bwie.common.domain.request.CouponReq;
import com.bwie.common.exception.ProductException;
import com.bwie.common.result.Result;
import com.bwie.coupon.service.CouponService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Author：weiran
 * @Package：com.bwie.coupon.controller
 * @Project：monthA8.12
 * @name：CouponController
 * @Date：2024/8/12 11:04
 */
@RestController
@RequestMapping("/coupon")
public class CouponController {
    @Autowired
    private CouponService couponService;


    /**
     * 展示券码列表
     * @param couponReq
     * @return
     */
    @PostMapping("/getList")
    public Result getList(@RequestBody CouponReq couponReq){
        return Result.success(couponService.getList(couponReq));
    }

    /**
     * 展示券码类型列表
     */
    @PostMapping("/getTypeList")
    public Result getTypeList(){
        return Result.success(couponService.getTypeList());
    }

    /**
     * 展示券码分组列表
     */
    @PostMapping("/gettypeListBytypeGroupname")
    public Result gettypeListBytypeGroupname(){
        return Result.success(couponService.gettypeListBytypeGroupname());
    }

    /**
     * 购买，生成订单
     * @param couponBuyReq
     * @return
     */
    @PostMapping("/buyyes")
    public Result buyyes(@RequestBody CouponBuyReq couponBuyReq){
        Integer buy = couponService.buy(couponBuyReq);
        if (buy>0){
            return Result.success();
        }
        throw new ProductException("生成订单出现异常");
    }


    /**
     * 查询我自己拥有券，来售卖
     */
    @PostMapping("/getmycouponList")
    public Result getmycouponList(){
       return Result.success(couponService.findMycoupon()) ;
    }

    /**
     * MQ进行异步更新ES索引库
     */
    @PostMapping("/updes")
    public void updes(){
        couponService.updes();
    }

    /**
     * 根据券码分组查找
     */
    @PostMapping("/choosetypeRroup/{typeGroupname}")
    public Result choosetypeRroup(@PathVariable String typeGroupname){
        return Result.success(couponService.choosetypeRroup(typeGroupname));
    }

}
