package com.bwie.coupon.controller;

import com.bwie.common.domain.Order;
import com.bwie.common.domain.request.CouponSellReq;
import com.bwie.common.exception.ProductException;
import com.bwie.common.result.Result;
import com.bwie.coupon.service.OrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Author：weiran
 * @Package：com.bwie.coupon.controller
 * @Project：monthA8.12
 * @name：OrderController
 * @Date：2024/8/12 15:54
 */
@RestController
@RequestMapping("/order")
public class OrderController {
    @Autowired
    private OrderService orderService;


    /**
     * 订单列表
     * @return
     */
    @PostMapping("/getList")
    public Result getList(){
        return Result.success(orderService.getList());
    }

    /**
     * 确认收货
     * @param order
     * @return
     */
    @PostMapping("/updateorderstatus")
    public Result updateorderstatus(@RequestBody Order order){
        Integer updateorderstatus = orderService.updateorderstatus(order);
        if (updateorderstatus>0){
            return Result.success();
        }
       throw new ProductException("确认收货异常，请稍后再试");
    }


    /**
     * 48小时未点击确认收货则自动确认收货
     */
    @Scheduled(cron = "* * 0/48 * * *")
    @PostMapping("/receive")
    public void receive(){
        orderService.receive();
    }


    /**
     * 售卖券码
     * @param couponSellReq
     * @return
     */
    @PostMapping("/sellyes")
    public Result sellyes(@RequestBody CouponSellReq couponSellReq){
        Integer sellyes = orderService.sellyes(couponSellReq);
        if (sellyes>0){
            return Result.success();
        }
        throw new ProductException("售卖异常，请稍后再试");
    }
}
