package com.bwie.coupon.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.nacos.common.utils.RandomUtils;
import com.bwie.common.domain.Coupon;
import com.bwie.common.domain.MyCoupon;
import com.bwie.common.domain.Order;
import com.bwie.common.domain.Type;
import com.bwie.common.domain.request.CouponBuyReq;
import com.bwie.common.domain.request.CouponReq;
import com.bwie.common.domain.request.CouponSellReq;
import com.bwie.common.domain.response.CouponResp;
import com.bwie.common.exception.ProductException;
import com.bwie.coupon.mapper.CouponMapper;
import com.bwie.coupon.service.CouponService;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.UUID;

/**
 * @Author：weiran
 * @Package：com.bwie.coupon.service.impl
 * @Project：monthA8.12
 * @name：CouponServiceImpl
 * @Date：2024/8/12 11:04
 */
@Service
public class CouponServiceImpl implements CouponService {
    @Autowired
    private CouponMapper couponMapper;
    @Autowired
    private RabbitTemplate rabbitTemplate;


    /**
     * 展示券码列表
     * @param couponReq
     * @return
     */
    @Override
    public List<CouponResp> getList(CouponReq couponReq) {
        List<CouponResp> list = couponMapper.getList(couponReq);
        return list;
    }


    /**
     * 展示券码类型列表
     */
    @Override
    public List<Type> getTypeList() {
        List<Type> typeList = couponMapper.getTypeList();
        return typeList;
    }


    /**
     * 展示券码分组列表
     */
    @Override
    public List<Type> gettypeListBytypeGroupname() {
        List<Type> types = couponMapper.gettypeListBytypeGroupname();
        return  types;
    }


    /**
     * 购买，生成订单
     * @param couponBuyReq
     * @return
     */
    @Override
    public Integer buy(CouponBuyReq couponBuyReq) {
        MyCoupon byname = couponMapper.findByname(couponBuyReq.getCouponName());
        if (byname!=null){
            throw new ProductException("您已购买完成，无法再次进行购买");
        }


        //生成兑换码
        String code = RandomUtil.randomNumbers(10);
        couponBuyReq.setExchangeNum(code);
        //生成订单
        Integer addorder = couponMapper.addorder(couponBuyReq);
        if (addorder>0){
            //自己的仓库中添加购买的券
            couponMapper.addmycoupon(couponBuyReq);

            //异步给客户扣款
            rabbitTemplate.convertAndSend("updatemoney", JSON.toJSONString(couponBuyReq), message->{
                message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                return message;
            });

            //生成订单成功后，库存-1
            couponMapper.reducecouponNum(couponBuyReq.getCouponId());
        }
        return addorder;
    }

    @Override
    public List<MyCoupon> findMycoupon() {
       return couponMapper.findMycoupon();
    }

    @Override
    public void del(CouponSellReq couponSellReq) {
        couponMapper.del(couponSellReq);
    }


    /**
     * MQ进行异步更新ES索引库
     */
    @Override
    public void updes() {
        List<Coupon> findall = couponMapper.findall();
        //MQ进行异步更新ES索引库
        rabbitTemplate.convertAndSend("updatees",findall, message->{
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        });
    }

    @Override
    public List<CouponResp> choosetypeRroup(String typeGroupname) {
        return couponMapper.choosetypeRroup(typeGroupname);
    }


}
