package com.bwie.coupon.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Order;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.CouponSellReq;
import com.bwie.common.domain.request.ReduceMoneyReq;
import com.bwie.common.utils.JwtUtils;
import com.bwie.coupon.feign.CouponFeign;
import com.bwie.coupon.mapper.OrderMapper;
import com.bwie.coupon.service.CouponService;
import com.bwie.coupon.service.OrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.List;
import java.util.Objects;

/**
 * @Author：weiran
 * @Package：com.bwie.coupon.service.impl
 * @Project：monthA8.12
 * @name：OrderServiceImpl
 * @Date：2024/8/12 15:54
 */
@Service
public class OrderServiceImpl implements OrderService {
    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private CouponFeign couponFeign;
    @Autowired
    private CouponService couponService;
    @Autowired
    private RedisTemplate<Object,Object> redisTemplate1;


    /**
     * 订单列表
     * @return
     */
    @Override
    public List<Order> getList() {
       return orderMapper.getList();
    }


    /**
     * 确认收货
     * @param order
     * @return
     */
    @Override
    public Integer updateorderstatus(Order order) {
        return orderMapper.updateorderstatus(order);
    }


    /**
     * 48小时未点击确认收货则自动确认收货
     */
    @Override
    public void receive() {
        //查询出48小时之后还未收到的信息
        List<Order> list = orderMapper.findBytime();
        for (Order order : list) {
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            LocalDateTime parse = LocalDateTime.parse(order.getOrderTime(), dateTimeFormatter);
            LocalDateTime now = LocalDateTime.now();
            long between = ChronoUnit.HOURS.between(parse, now);
            if (between>24){
                orderMapper.updateorderstatus(order);
            }
        }


    }

    /**
     * 售卖券码
     * @param couponSellReq
     * @return
     */
    @Override
    public Integer sellyes(CouponSellReq couponSellReq) {
        //生成订单
        Integer add = orderMapper.add(couponSellReq);
        if (add>0){
            //用户余额加款
            String token = request.getHeader("token");
            String userKey = JwtUtils.getUserKey(token);
            String code = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
            User user = JSONObject.parseObject(code, User.class);
            ReduceMoneyReq reduceMoneyReq = new ReduceMoneyReq(user.getUserId(), couponSellReq.getMoney());
           couponFeign.addmoney(reduceMoneyReq);

           //把售卖的券码删除
            couponService.del(couponSellReq);
        }
        List<Order> findorder = orderMapper.findorder();
        for (Order order : findorder) {
            List<Object> range = redisTemplate1.opsForList().range(order.getOrderId(), 0, -1);
            if (range==null){
                redisTemplate1.opsForSet().add(order.getOrderId(), order);
            }
        }

        return  add;
    }
}
