package com.bwie.es.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.Coupon;
import com.bwie.common.domain.request.CouponReq;
import com.bwie.common.domain.response.CouponResp;
import com.bwie.common.utils.StringUtils;
import com.bwie.es.service.EsService;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author：weiran
 * @Package：com.bwie.es.service.impl
 * @Project：monthA8.12
 * @name：EsServiceImpl
 * @Date：2024/8/12 11:49
 */
@Service
public class EsServiceImpl implements EsService {
    @Autowired
    private RestHighLevelClient restHighLevelClient;

    private static final String INDEX_NAME="coupon";
    /**
     * 批量添加
     * @param coupons
     */
    @Override
    public void bathadd(List<Coupon> coupons) {
        BulkRequest bulkRequest = new BulkRequest();
        for (Coupon coupon : coupons) {
            bulkRequest.add(new IndexRequest(INDEX_NAME)
                    .id(coupon.getCouponId()+"")
                    .source(JSONObject.toJSONString(coupon), XContentType.JSON));
        }
        try {
            restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * 查询
     */
    @Override
    public List<CouponResp> search(CouponReq couponReq) {
        ArrayList<CouponResp> couponResps = new ArrayList<>();
        try {
            SearchRequest searchRequest = new SearchRequest(INDEX_NAME);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
            if (StringUtils.isNotBlank(couponReq.getCouponName())){
                boolQueryBuilder.must(QueryBuilders.matchQuery("couponName", couponReq.getCouponName()));
            }
            if (couponReq.getTypeId()!=null){
                boolQueryBuilder.must(QueryBuilders.matchQuery("typeId", couponReq.getTypeId()));
            }
            searchSourceBuilder.query(boolQueryBuilder);
            searchSourceBuilder.highlighter(
                    new HighlightBuilder()
                            .field("couponName")
                            .preTags("<span style=\"color:red;\">")
                            .postTags("</span>")
            );
            searchRequest.source(searchSourceBuilder);
            SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = searchResponse.getHits();
            SearchHit[] searchHits = hits.getHits();
            for (SearchHit searchHit : searchHits) {
                if (searchHit!=null){
                    String sourceAsString = searchHit.getSourceAsString();
                    CouponResp couponResp = JSONObject.parseObject(sourceAsString, CouponResp.class);
                    Map<String, HighlightField> highlightFields = searchHit.getHighlightFields();

                    HighlightField highlightField = highlightFields.get("couponName");
                    Text[] fragments = highlightField.fragments();
                    String str="";
                    for (Text fragment : fragments) {
                        str+=fragment;
                        couponResp.setCouponName(str);
                    }

                    couponResps.add(couponResp);
                }

            }

            return couponResps;

        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }


}
