package com.bwie.mq.consumer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.CouponBuyReq;
import com.bwie.common.domain.request.ReduceMoneyReq;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.mq.feign.MqFeign;
import com.rabbitmq.client.Channel;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;


/**
 * @Author：weiran
 * @Package：com.bwie.mq.consumer
 * @Project：monthA8.12
 * @name：UpdateMoney
 * @Date：2024/8/12 16:18
 */
@Component
public class UpdateMoney {
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private MqFeign mqFeign;


    @RabbitListener(queuesToDeclare = @Queue("updatemoney"))
    public void UpdateMoney(String coupons , Message message, Channel channel){
        CouponBuyReq couponBuyReq = JSON.parseObject(coupons, CouponBuyReq.class);
        String messageId = message.getMessageProperties().getMessageId();
        Long userKey1 = redisTemplate.opsForSet().add("UPDATEMONEY", messageId);
        if (userKey1>0 && userKey1==1){
            String token = request.getHeader("token");
            String userKey = JwtUtils.getUserKey(token);
            String code = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
            User user = JSONObject.parseObject(code, User.class);
            ReduceMoneyReq reduceMoneyReq = new ReduceMoneyReq(user.getUserId(), couponBuyReq.getCouponPrice());
            Result changemoney = mqFeign.changemoney(reduceMoneyReq);
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            } catch (IOException e) {
                try {
                    redisTemplate.opsForSet().remove("UPDATEMONEY", messageId);
                    channel.basicAck(message.getMessageProperties().getDeliveryTag(),true);
                } catch (IOException ex) {
                    throw new RuntimeException(ex);
                }
                throw new RuntimeException(e);
            }

        }
    }
}
