package com.bwie.user.controller;

import com.bwie.common.domain.User;
import com.bwie.common.domain.request.ReduceMoneyReq;
import com.bwie.common.domain.request.UserReq;
import com.bwie.common.domain.response.JwtResp;
import com.bwie.common.result.Result;
import com.bwie.user.service.UserService;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Author：weiran
 * @Package：com.bwie.user.controller
 * @Project：monthA8.12
 * @name：UserController
 * @Date：2024/8/12 9:36
 */
@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    private UserService userService;


    /**
     * 根据手机号查询用户信息
     * @param userPhone
     * @return
     */
    @PostMapping("/user/findByuserPhone/{userPhone}")
    Result<User> findByuserPhone(@PathVariable String userPhone){
        return Result.success(userService.findByuserPhone(userPhone));
    }

    /**
     * 录入用户信息
     * @param user
     * @return
     */
    @PostMapping("/user/adduser")
    Result adduser(@RequestBody User user){
        return Result.success(userService.adduser(user));
    }

    /**
     *MQ异步给客户扣款
     */
    @PostMapping("/user/changemoney")
    Result changemoney(@RequestBody ReduceMoneyReq reduceMoneyReq){
        Integer changemoney = userService.changemoney(reduceMoneyReq);
        if (changemoney>0){
            return Result.success();
        }
        return Result.error();
    }

    /**
     * 售卖券码后给用户加钱
     * @param reduceMoneyReq
     * @return
     */
    @PostMapping("/user/addmoney")
    Result addmoney(@RequestBody ReduceMoneyReq reduceMoneyReq){
        Integer addmoney = userService.addmoney(reduceMoneyReq);
        if (addmoney>0){
            return Result.success();
        }
        return Result.error();
    }
}
